@@ -1,16 +1,16 @@
 /***************************************************************************
- * Unit:    raster            release 0.25                                 *
+ * Unit:    raster            release 0.26                                 *
  * Purpose: Prototypes for general manipulation n dimensional matrices     *
  * Modul:   raster.cc                                                      *
  * Licency: GPL or LGPL                                                    *
- * Copyright: (c) 1998-2023 Jaroslav Fojtik                                *
+ * Copyright: (c) 1998-2024 Jaroslav Fojtik                                *
  ***************************************************************************/
 #ifndef __Rasters_h
 #define __Rasters_h
 
 #include "typedfs.h"
 
-#define RASTER_VERSION (0x100*0 | 25)
+#define RASTER_VERSION (0x100*0 | 26)
 
 #ifndef No_Memory
  #define No_Memory    0x1
@@ -151,6 +151,7 @@ public: mutable short UsageCount;
 	virtual double GetValue1Dd(unsigned x) const {return(GetValue1D(x));};
 	virtual void SetValue1D(unsigned x, uint32_t NewValue) ABSTRACT_ERROR("SetValue1D");
 	virtual void SetValue1Dd(unsigned x, double NewValue) {SetValue1D(x,(uint32_t)NewValue);};
+        virtual void Set(unsigned x, const RGBQuad *RGB) {SetValue1D(x,(RGB->R+RGB->G+RGB->B)/3);};
 	virtual void Get(unsigned x, RGBQuad *RGB) const {RGB->O=0;RGB->R=RGB->G=RGB->B=GetValue1D(x);};
 
 	virtual int GetPlanes(void) const {return(0);};
@@ -203,9 +204,9 @@ public:
   virtual void Get(unsigned x, RGBQuad *RGB) const;
   virtual void Set(unsigned x, const RGBQuad *RGB);
 
-  uint32_t R(unsigned x) const {return(GetValue1DRAW(3*x));}
-  uint32_t G(unsigned x) const {return(GetValue1DRAW(3*x+1));}
-  uint32_t B(unsigned x) const {return(GetValue1DRAW(3*x+2));}
+  virtual uint32_t R(unsigned x) const {return(GetValue1DRAW(3*x));}
+  virtual uint32_t G(unsigned x) const {return(GetValue1DRAW(3*x+1));}
+  virtual uint32_t B(unsigned x) const {return(GetValue1DRAW(3*x+2));}
 
   void setR(unsigned x, uint32_t newR) {SetValue1DRAW(3*x,newR);};
   void setG(unsigned x, uint32_t newG) {SetValue1DRAW(3*x+1,newG);};
@@ -281,6 +282,7 @@ public: unsigned Size2D;
 	void *GetRow(unsigned Offset2D)   {if(Data2D==NULL || Offset2D>=Size2D) return(NULL);return(Data2D[Offset2D]);}
         const void *GetRow(unsigned Offset2D) const {if(Data2D==NULL || Offset2D>=Size2D) return(NULL);return(Data2D[Offset2D]);}
         virtual void Get(unsigned Offset1D, unsigned Offset2D, RGBQuad *RGB);
+        virtual void Set(unsigned Offset1D, unsigned Offset2D, const RGBQuad *RGB);
 
 	Raster1DAbstract *GetRowRaster(unsigned Offset2D);
 
@@ -469,6 +471,7 @@ void Flip1D(Raster1DAbstract *r1D);
 
 void Flip1D(Raster2DAbstract *r2D);
 void Flip2D(Raster2DAbstract *r2D);
+Raster2DAbstract *Flip1D2D(Raster2DAbstract *r2D);
 int GrayPalette(APalette *Palette, int Planes=0);
 int FillGray(Raster1DAbstractRGB *Palette, int Planes=0);
 
