@@ -1,9 +1,9 @@
 /******************************************************************************
- * program:     rasimg library 0.23                                           *
+ * program:     rasimg library 0.26                                           *
  * function:    Object set for handling vector images.                        *
  * modul:       vec_image.cc                                                  *
  * licency:     GPL or LGPL                                                   *
- * Copyright: (c) 2018-2023 Jaroslav Fojtik                                   *
+ * Copyright: (c) 2018-2024 Jaroslav Fojtik                                   *
  ******************************************************************************/
 #include <stdlib.h>
 #include <stdio.h>
@@ -345,7 +345,7 @@ void vecBrush::AttribFromPSS(const PS_State &PSS)
 {
   memcpy(&FillColor, &PSS.FillColor, sizeof(FillColor));
   memcpy(&FillBackground, &PSS.FillBackground, sizeof(FillBackground));
-  BrushStyle = PSS.FillPattern;  
+  BrushStyle = PSS.FillPattern;
 }
 
 
@@ -765,10 +765,17 @@ return temp_string(str);
 }
 
 
-void VectorRectangle::Transform(const AbstractTransformXY &Tx)
+void VectorRectangle::Transform(const AbstractTransformXY &NewTx)
 {
-  Tx.ApplyTransform(LeftRect, BottomRect);
-  Tx.ApplyTransform(RightRect, TopRect);
+  if(Tx)	// Tx must be flushed, transformations are not comutative.
+  {
+    Tx->ApplyTransform(LeftRect, BottomRect);
+    Tx->ApplyTransform(RightRect, TopRect);
+    delete Tx;
+    Tx = NULL;
+  }
+  NewTx.ApplyTransform(LeftRect, BottomRect);
+  NewTx.ApplyTransform(RightRect, TopRect);
 
   if(TopRect < BottomRect)
   {
@@ -920,6 +927,16 @@ string str;
       }
     }
   }
+
+  if(LineStyle==0 && BrushStyle==0)	// No line and no fill means invisible object,
+  {
+    return 
+#ifdef _DEBUG
+	"\n%%Polygon skipped because it is not visible.\n";
+#else
+	"";			// Skip it.   
+#endif
+  }
   
   const float *pEnd = Points + 2*CountPoints;
   float MinPolyX=65537,MaxPolyX=-32767,MinPolyY=65537,MaxPolyY=-32767;
@@ -1871,7 +1888,7 @@ string Override;
   {    
     if(PSS!=NULL)
     {    
-      PS_Attr(str,PSS);   
+      PS_Attr(str,PSS);
       memcpy(&BkTextColor,&PSS->LineColor,sizeof(RGB_Record));
       if(MixMode==2)
           Override.printf("\ngsave dup"
@@ -2134,7 +2151,7 @@ TextObject *pTobj;
     pTobj->contents = contents;
     pTobj->size = PSS.FontSize;
     //pTobj->Weight = Weight;
-    memcpy(&pTobj->TextColor,&PSS.TextColor,sizeof(RGB_Record));  
+    memcpy(&pTobj->TextColor,&PSS.TextColor,sizeof(RGB_Record));
     TextObjects = 1;
     return;
   }
@@ -2161,7 +2178,7 @@ TextObject *pTobj;
   
   pTobj->size = PSS.FontSize;
   //pTobj->Weight = Weight;  
-  memcpy(&pTobj->TextColor,&PSS.TextColor,sizeof(RGB_Record));  
+  memcpy(&pTobj->TextColor,&PSS.TextColor,sizeof(RGB_Record));
 }
 
 
@@ -2412,9 +2429,9 @@ float Distance;
     PSS->FirstTimeFix = false;
     }
 
-  if(PSS->FillPattern > (0x80|FILL_SOLID))
+  if(PSS->FillPattern >= (0x80|FILL_SOLID))
   {
-    color.printf("\n%0.2g %0.2g %0.2g setrgbcolor",
+     color.printf("\n%0.2g %0.2g %0.2g setrgbcolor",
 		(float)PSS->FillBackground.Red/256.0,
 	 	(float)PSS->FillBackground.Green/256.0,
 		(float)PSS->FillBackground.Blue/256.0);
