@@ -1,9 +1,9 @@
 /***************************************************************************
- * Unit:    rasimg library 0.25                                            *
+ * Unit:    rasimg library 0.26                                            *
  * Purpose: Prototypes for general manipulation vith vector images         *
  * Modul:   vecimage.h                                                     *
  * Licency: GPL or LGPL                                                    *
- * Copyright: (c) 1998-2023 Jaroslav Fojtik                                *
+ * Copyright: (c) 1998-2024 Jaroslav Fojtik                                *
  ***************************************************************************/
 #ifndef _VECIMAGE_H_
 #define _VECIMAGE_H_
@@ -383,6 +383,10 @@ public:
 
   virtual temp_string Export2EPS(PS_State *PSS=NULL) const;
   void Export2EPS(FILE *F, PS_State *PSS) const;
+#ifdef _DEBUG
+  virtual void Dump2File(FILE *F) const {fprintf(F,"VectorPath\n");VectorList::Dump2File(F);fprintf(F,"~VectorPath\n");}
+#endif
+
 };
 
 
@@ -486,7 +490,7 @@ public:
 
   virtual void CalcBoundingBox(FloatBBox &bbx) const;
   virtual temp_string Export2EPS(PS_State *PSS=NULL) const;
-  virtual void Transform(const AbstractTransformXY &Tx);
+  virtual void Transform(const AbstractTransformXY &NewTx);
   virtual unsigned isInside(float xp, float yp) const;
   void AttribFromPSS(const PS_State &PSS) {vecPen::AttribFromPSS(PSS);vecBrush::AttribFromPSS(PSS);}
   virtual VectorObject *Duplicate(void) const;
