@@ -6,7 +6,8 @@
  *              It could be optionally not compiled with WP2LaTeX package.    *
  * licency:     GPL		                                              *
  ******************************************************************************/
-#define RTF_Ver "0.16"
+#define RTF_Ver "0.18"
+// More info about RTF: https://www.biblioscape.com/rtf15_spec.htm
 //TODO: Dodelat znak ? - maze mezeru pred sebou
 //      Autodetekce \section podle xref _TOCxxxxxx
 
@@ -116,10 +117,10 @@ typedef enum {kwdChar, kwdDest, kwdProp, kwdSpec} KWD;
 
 typedef struct
    {
-   ACTN PropSize:2;        	// size of value
-   KWD  Kwd:2;   		// base action to take
-   PROPTYPE PropScope:2;        // scope - structure containing value
-   unsigned char ForceArg:1; 	// true to use default value from this table
+   ACTN PropSize:2;		// size of value
+   KWD  Kwd:2;			// base action to take
+   PROPTYPE PropScope:2;	// scope - structure containing value
+   unsigned char ForceArg:1;	// true to use default value from this table
    }TFlags;
 
 
@@ -183,6 +184,10 @@ public:CpTranslator *Unicode;
        void SpecialObject(void);
        void fldinst(void);
        int ProcessKeyRTF(void);
+       int RTFLoadKeyword(void);
+       void HeaderFooterRTF(unsigned char HFtype);
+       void IndexRTF(void);
+       void ScanTableFormat(void);
        };
 TconvertedPass1 *Factory_RTF(void) {return new TconvertedPass1_RTF;}
 FFormatTranslator FormatRTF("RTF",Factory_RTF);
@@ -583,7 +588,7 @@ int isym;
 // found it!  use kwd and idx to determine what to do with it.
 
   fSkipDestIfUnk = false;
-  switch (rgsymRtf[isym].F.Kwd & 3)
+  switch(rgsymRtf[isym].F.Kwd & 3)
     {
     case kwdProp:
 	if (rgsymRtf[isym].F.ForceArg || !fParam)
@@ -744,23 +749,23 @@ int param = 0;
 
 //returns 0 - EOF;1-Binary;2-unknown;3-Bad Hex
 //        0xA;0xD - EOL;200-{;201-};202-keyword no arg;203-wrong arg;204-keyword+arg;205-character;206-HEX
-static int RTFLoadKeyword(TconvertedPass1_RTF *cq)
+int TconvertedPass1_RTF::RTFLoadKeyword(void)
 {
 #ifdef DEBUG
-  fprintf(cq->log,"\n#RTFLoadKeyword() ");fflush(cq->log);
+  fprintf(log,"\n#TconvertedPass1_RTF::RTFLoadKeyword() ");fflush(log);
 #endif
 int ch;
 
- cq->Parameter.erase();
- cq->Keyword.erase();
- cq->lParam=0;
- cq->isParam=false;
- if ((ch = getc(cq->wpd)) == EOF) return(0); //no Keyword
- cq->by=ch;
+ Parameter.erase();
+ Keyword.erase();
+ lParam = 0;
+ isParam = false;
+ if((ch = getc(wpd)) == EOF) return(0); //no Keyword
+ by = ch;
 
- if (cq->ris == risBin)
+ if(ris == risBin)
 	{
-	cq->Keyword=(char)ch;
+	Keyword=(char)ch;
 	return(1);	// if we're parsing binary data, handle it directly
 	}
 
@@ -771,79 +776,79 @@ int ch;
 
  if(ch=='\\')
 	{
-	cq->Keyword.erase();
-        cq->Parameter.erase();
-	if ((ch = getc(cq->wpd)) == EOF) return(0);
+	Keyword.erase();
+        Parameter.erase();
+	if ((ch = getc(wpd)) == EOF) return(0);
 	if (ch == '\'')
 	     {
-	     ch=getc(cq->wpd);
+	     ch=getc(wpd);
 	     if(!isxdigit(ch)) {return(3);}
 	     ch = toupper(ch);
 	     if(ch>='A') ch-='A'-10;
 		    else ch-='0';
-	     cq->by=ch;
-	     ch=getc(cq->wpd);
+	     by=ch;
+	     ch=getc(wpd);
 	     if(!isxdigit(ch)) {return(3);}
 	     ch = toupper(ch);
 	     if(ch>='A') ch-='A'-10;
 		    else ch-='0';
-	     cq->by=16*cq->by+ch;
+	     by=16*by+ch;
 	     return(206);
 	     }
 	if (!isalpha(ch))           // a control symbol; no delimiter.
 	     {
-	     cq->Keyword = (char) ch;
-	     return(202);  // ecTranslateKeyword(cq,cq->Keyword(), 0, fParam);
+	     Keyword = (char) ch;
+	     return(202);  // ecTranslateKeyword(cq,Keyword(), 0, fParam);
 	     }
 	while(isalpha(ch))
 	       {
-	       cq->Keyword += (char) ch;
-	       ch = getc(cq->wpd);
+	       Keyword += (char) ch;
+	       ch = getc(wpd);
 	       }
 	if (ch == '-')
 	      {
-	      cq->Parameter = (char) ch;
-	      if ((ch = getc(cq->wpd)) == EOF)
+	      Parameter = (char) ch;
+	      if ((ch = getc(wpd)) == EOF)
 		   {
-		   ungetc(*cq->Parameter(), cq->wpd);
-		   cq->Parameter.erase();
+		   ungetc(*Parameter(), wpd);
+		   Parameter.erase();
 		   return(203);		//unexpected EOF
 		   }
 	      }
 	if(isdigit(ch))
 	     {
-	     cq->isParam = true;         // a digit after the control means we have a parameter
+	     isParam = true;         // a digit after the control means we have a parameter
 	     while(isdigit(ch))
 		{
-		cq->Parameter += (char) ch;
-		ch = getc(cq->wpd);
+		Parameter += (char) ch;
+		ch = getc(wpd);
 		}
-	     cq->fParam = atoi(cq->Parameter());
-	     cq->lParam = atol(cq->Parameter());
+	     fParam = atoi(Parameter());
+	     lParam = atol(Parameter());
 	     }
 	if(ch != ' ')
-	       ungetc(ch, cq->wpd);
-	return(204); // ecTranslateKeyword(cq,cq->Keyword(), param, fParam);
+	       ungetc(ch, wpd);
+	return(204); // ecTranslateKeyword(cq,Keyword(), param, fParam);
 	}
 
- if (cq->ris == risNorm) return(205);	//if ((ec = ecParseChar(cq,ch)) != ecOK)
+ if(ris == risNorm) return(205);	//if ((ec = ecParseChar(cq,ch)) != ecOK)
 
- if (cq->ris != risHex) return(2);
+ if(ris != risHex) return(2);
 
- if (isdigit(ch)) cq->by = (char) ch - '0';
+ if (isdigit(ch)) by = (char) ch - '0';
  else {
-     if (ch >= 'a' && ch <= 'f') cq->by = (char) ch - 'a';
-	 else if (ch >= 'A' && ch <= 'F') cq->by = (char) ch - 'A';
+     if (ch >= 'a' && ch <= 'f') by = (char) ch - 'a';
+	 else if (ch >= 'A' && ch <= 'F') by = (char) ch - 'A';
 	      else return(3);
      }
- if ((ch = getc(cq->wpd)) == EOF) return(206);
- cq->by = cq->by << 4;
- if (isdigit(ch)) cq->by |= (char) ch - '0';
+ if ((ch = getc(wpd)) == EOF) return(206);
+ by = by << 4;
+ if (isdigit(ch)) by |= (char) ch - '0';
  else {
-      if (ch >= 'a' && ch <= 'f') cq->by |= (char) ch - 'a';
-	 else if (ch >= 'A' && ch <= 'F') cq->by |= (char) ch - 'A';
+      if (ch >= 'a' && ch <= 'f') by |= (char) ch - 'a';
+	 else if (ch >= 'A' && ch <= 'F') by |= (char) ch - 'A';
 	      else {
-		   ungetc(ch, cq->wpd);
+		   ungetc(ch, wpd);
 		   return(206);
 		   }
       }
@@ -889,7 +894,7 @@ RGBQuad RGB;
  RGB.O=0;
  while (!feof(cq->wpd))
 	{
-	switch(cq->subby=RTFLoadKeyword(cq))
+	switch(cq->subby=cq->RTFLoadKeyword())
 	   {
 	   case 0xA:break;
 	   case 0xD:break;
@@ -1031,7 +1036,7 @@ static void FootnoteRTF(TconvertedPass1_RTF *cq)
   cq->attr.InitAttr();
 
   cq->ActualPos = ftell(cq->wpd);
-  cq->subby=RTFLoadKeyword(cq);
+  cq->subby = cq->RTFLoadKeyword();
   if(cq->subby==204 && cq->Keyword=="ftnalt")
 	{
 	if(!EndNotes) EndNotes=true;		/* set up endnotes */
@@ -1045,7 +1050,7 @@ static void FootnoteRTF(TconvertedPass1_RTF *cq)
 
   while (!feof(cq->wpd) && CurlyBaraceStack > 0)
 	{
-	cq->subby=RTFLoadKeyword(cq);
+	cq->subby = cq->RTFLoadKeyword();
 	if(cq->subby==200) CurlyBaraceStack++;
 	if(cq->subby==201) CurlyBaraceStack--;
 
@@ -1107,7 +1112,7 @@ static void FontTableRTF(TconvertedPass1_RTF *cq)
 
   while (!feof(cq->wpd) && CurlyBaraceStack > 0)
 	{
-	cq->subby=RTFLoadKeyword(cq);
+	cq->subby = cq->RTFLoadKeyword();
 	if(cq->subby==200) CurlyBaraceStack++;
 	if(cq->subby==201) CurlyBaraceStack--;
 
@@ -1124,7 +1129,7 @@ static void FontTableRTF(TconvertedPass1_RTF *cq)
 
 		while (!feof(cq->wpd) && CurlyBaraceStack>=StackDepth)
 		  {
-		  cq->subby=RTFLoadKeyword(cq);
+		  cq->subby = cq->RTFLoadKeyword();
 		  switch(cq->subby)
 		    {
 		    case 0:   break;
@@ -1160,121 +1165,121 @@ static void FontTableRTF(TconvertedPass1_RTF *cq)
 }
 
 
-static void HeaderFooterRTF(TconvertedPass1_RTF *cq, unsigned char HFtype)
+void TconvertedPass1_RTF::HeaderFooterRTF(unsigned char HFtype)
 {
 #ifdef DEBUG
-  fprintf(cq->log,"\n#HeaderFooterRTF(%d) ",(int)HFtype); fflush(cq->log);
+  fprintf(log,"\n#TconvertedPass1_RTF::HeaderFooterRTF(%d) ",(int)HFtype); fflush(log);
 #endif
   unsigned char OldFlag, OldEnvir, OldFont;
   attribute OldAttr;
   int CurlyBaraceStack;
 
   CurlyBaraceStack = 1;
-  OldFlag = cq->flag;
+  OldFlag = flag;
 
-  Close_All_Attr(cq->attr, cq->strip);
-  OldAttr = cq->attr;
-  OldEnvir = cq->envir;
-  OldFont = cq->Font;
-  cq->attr.InitAttr();		//Turn all attributes in the header/footer off
+  Close_All_Attr(attr, strip);
+  OldAttr = attr;
+  OldEnvir = envir;
+  OldFont = Font;
+  attr.InitAttr();		//Turn all attributes in the header/footer off
 
 // Start a conversion of a Header or Footer
-  cq->recursion++;
+  recursion++;
 
-  cq->line_term = 's';    	//Soft return
-  if(cq->char_on_line == LEAVE_ONE_EMPTY_LINE)  // Left one enpty line for new enviroment.
+  line_term = 's';    	//Soft return
+  if(char_on_line == LEAVE_ONE_EMPTY_LINE)  // Left one enpty line for new enviroment.
       {
-      fputc('%', cq->strip);
-      NewLine(cq);
+      fputc('%', strip);
+      NewLine(this);
       }
-  if(cq->char_on_line==CHAR_PRESENT)
+  if(char_on_line==CHAR_PRESENT)
       {
-      NewLine(cq);
+      NewLine(this);
       }
 
   if(OldEnvir=='L' || OldEnvir=='B') // || BkEnvir=='i')
   {
-    cq->envir = '!';		//Ignore enviroments after header/footer
-    NewLine(cq);
-    cq->envir = ' ';
+    envir = '!';		//Ignore enviroments after header/footer
+    NewLine(this);
+    envir = ' ';
   }
-  //cq->envir = ' ';
+  //envir = ' ';
 
 //	             (HFtype2 & 3) <= 1 ? "\\headtext":"\\foottext"
-  InitHeaderFooter(cq,HFtype & 3,HFtype >> 2);
+  InitHeaderFooter(this,HFtype & 3,HFtype >> 2);
 
-  cq->flag = HeaderText;
-  cq->char_on_line = FIRST_CHAR_MINIPAGE;
-  while (!feof(cq->wpd) && CurlyBaraceStack > 0)
+  flag = HeaderText;
+  char_on_line = FIRST_CHAR_MINIPAGE;
+  while (!feof(wpd) && CurlyBaraceStack > 0)
 	{
-	cq->subby=RTFLoadKeyword(cq);
-	if(cq->subby==200) CurlyBaraceStack++;
-	if(cq->subby==201) CurlyBaraceStack--;
+	subby = RTFLoadKeyword();
+	if(subby==200) CurlyBaraceStack++;
+	if(subby==201) CurlyBaraceStack--;
 
-	if(cq->subby==204)
+	if(subby==204)
 	   {
-	   if(cq->Keyword=="par" || cq->Keyword=="pard")
+	   if(Keyword=="par" || Keyword=="pard")
 		{
-		putc(' ', cq->strip);
+		putc(' ', strip);
 		continue;
 		}
 	   }
 
-	switch (cq->by)
+	switch (by)
 	     {
 /*	     case 0xa:
-	     case 0xc:fprintf(cq->strip, "\\\\ ");
+	     case 0xc:fprintf(strip, "\\\\ ");
 		      break;*/
 
 	     case 0xb:
-	     case 0xd:putc(' ', cq->strip);
+	     case 0xd:putc(' ', strip);
 		      break;
 
-	     default: cq->ProcessKeyRTF();
+	     default: ProcessKeyRTF();
 		      break;
 	     }
 
 	}
-  cq->ActualPos = ftell(cq->wpd);
+  ActualPos = ftell(wpd);
 
-  Close_All_Attr(cq->attr,cq->strip);
+  Close_All_Attr(attr,strip);
 
-  if(cq->envir != ' ')
+  if(envir != ' ')
      {
-     cq->line_term = 's';    	//Soft return
-     NewLine(cq);
+     line_term = 's';    	//Soft return
+     NewLine(this);
      }
-  putc('}', cq->strip);
+  putc('}', strip);
 
-  if(cq->envir != ' ')
+  if(envir != ' ')
     {
-    cq->envir = ' ';
-    NewLine(cq);
+    envir = ' ';
+    NewLine(this);
     }
   if(OldEnvir=='L' || OldEnvir=='B') // || BkEnvir=='i')
   {
-    cq->line_term = 's';    	//Soft return
-    cq->envir = '^';		//Ignore enviroments Before
-    NewLine(cq);
-    cq->char_on_line = JUNK_CHARS;
+    line_term = 's';    	//Soft return
+    envir = '^';		//Ignore enviroments Before
+    NewLine(this);
+    char_on_line = JUNK_CHARS;
   }
 
-  cq->recursion--;
+  recursion--;
 
-  cq->attr = OldAttr;
-  cq->flag = OldFlag;
-  cq->envir= OldEnvir;
-  cq->Font = OldFont;
-  cq->char_on_line = FIRST_CHAR_MINIPAGE;	// stronger false;
+  attr = OldAttr;
+  flag = OldFlag;
+  envir= OldEnvir;
+  Font = OldFont;
+  char_on_line = FIRST_CHAR_MINIPAGE;	// stronger false;
 
-  strcpy(cq->ObjType, ((HFtype & 3) <= 1)?"Header":"Footer");
+  strcpy(ObjType, ((HFtype & 3) <= 1)?"Header":"Footer");
 }
 
 
-static void IndexRTF(TconvertedPass1_RTF *cq)
+void TconvertedPass1_RTF::IndexRTF(void)
 {
 #ifdef DEBUG
-  fprintf(cq->log,"\n#IndexRTF() ");fflush(cq->log);
+  fprintf(log,"\n#IndexRTF() ");fflush(log);
 #endif
   attribute OldAttr;
   int CurlyBaraceStack;
@@ -1282,62 +1287,152 @@ static void IndexRTF(TconvertedPass1_RTF *cq)
 
   CurlyBaraceStack=1;
 
-  OldFlag = cq->flag;
-  cq->flag = HeaderText;
-  cq->recursion++;
+  OldFlag = flag;
+  flag = HeaderText;
+  recursion++;
 
-  Close_All_Attr(cq->attr,cq->strip);
-  OldAttr=cq->attr;
-  cq->attr.InitAttr();
+  Close_All_Attr(attr,strip);
+  OldAttr=attr;
+  attr.InitAttr();
 
-  cq->ActualPos = ftell(cq->wpd);
-  fputs(" \\index{", cq->strip);
+  ActualPos = ftell(wpd);
+  fputs(" \\index{", strip);
 
-  while (!feof(cq->wpd) && CurlyBaraceStack > 0)
+  while (!feof(wpd) && CurlyBaraceStack > 0)
 	{
-	cq->subby=RTFLoadKeyword(cq);
-	if(cq->subby==200) CurlyBaraceStack++;
-	if(cq->subby==201) CurlyBaraceStack--;
+	subby = RTFLoadKeyword();
+	if(subby==200) CurlyBaraceStack++;
+	if(subby==201) CurlyBaraceStack--;
 
-	if(cq->subby==202 || cq->subby==203 || cq->subby==204)
+	if(subby==202 || subby==203 || subby==204)
 	   {
-	   if(cq->Keyword=="par" || cq->Keyword=="pard")
+	   if(Keyword=="par" || Keyword=="pard")
 		{
-		putc(' ', cq->strip);
+		putc(' ', strip);
 		continue;
 		}
 	   continue;
 	   }
 
-	switch (cq->by)
+	switch (by)
 	     {
 /*	     case 0xa:
-	     case 0xc:fprintf(cq->strip, "\\\\ ");
+	     case 0xc:fprintf(strip, "\\\\ ");
 		      break;*/
 
 	     case 0xb:
-	     case 0xd:putc(' ', cq->strip);
+	     case 0xd:putc(' ', strip);
 		      break;
 
-	     default: cq->ProcessKeyRTF();
+	     default: ProcessKeyRTF();
 		      break;
 	     }
 
 	   }
 
-  cq->ActualPos = ftell(cq->wpd);
-  Close_All_Attr(cq->attr,cq->strip);
-  putc('}', cq->strip);
+  ActualPos = ftell(wpd);
+  Close_All_Attr(attr,strip);
+  putc('}', strip);
   Index=true;
 
-  cq->attr=OldAttr;
+  attr=OldAttr;
 
-  cq->recursion--;
-  strcpy(cq->ObjType, "Index");
-  cq->flag = OldFlag;
+  recursion--;
+  strcpy(ObjType, "Index");
+  flag = OldFlag;
 }
 
 
+void TconvertedPass1_RTF::ScanTableFormat(void)
+{
+#ifdef DEBUG
+  fprintf(log,"\n#ScanTableFormat() ");fflush(log);
+#endif
+  attribute OldAttr;
+  int CurlyBaraceStack;
+  unsigned char OldFlag;
+  long BackupPos;
+  int cols = 0;
+  char *FieldPos = NULL;
+
+  CurlyBaraceStack = 1;
+
+  OldFlag = flag;
+  flag = Nothing;
+  recursion++;
+
+  BackupPos = ftell(wpd);
+/*
+  Close_All_Attr(attr,strip);
+  OldAttr=attr;
+  attr.InitAttr();
+
+  ActualPos = ftell(wpd);
+  fputs(" \\index{", strip);
+*/
+
+  FieldPos = (char*)calloc(100,1);
+  if(FieldPos==NULL) goto SkipParsing;
+
+  while(!feof(wpd) && CurlyBaraceStack > 0)
+  {
+    subby = RTFLoadKeyword();
+    if(subby==200) CurlyBaraceStack++;
+    if(subby==201) CurlyBaraceStack--;
+
+    if(subby==202 || subby==203 || subby==204)
+    {
+      if(Keyword=="par" || Keyword=="pard" ||
+         Keyword=="intbl" || Keyword=="cell" )
+      {
+        break;
+      }
+
+      if(cols<100)
+      {
+        if(Keyword=="trql")
+        {
+          FieldPos[cols] = 'l';
+          continue;
+        }
+        if(Keyword=="trqr")
+        {
+          FieldPos[cols] = 'r';
+          continue;
+        }
+        if(Keyword=="trqc")
+        {
+          FieldPos[cols] = 'c';
+          continue;
+        }
+      }
+
+      if(Keyword=="cellx")
+      {
+        cols++;
+        continue;
+      }
+      continue;
+    }
+  }
+/*
+  ActualPos = ftell(wpd);
+  Close_All_Attr(attr,strip);
+  putc('}', strip);
+  Index=true;
+
+  attr=OldAttr;
+*/
+SkipParsing:
+  fseek(wpd, BackupPos, SEEK_SET);
+
+  recursion--;
+  if(FieldPos) {free(FieldPos);FieldPos=NULL;}
+
+  strcpy(ObjType, "trowd");
+  flag = OldFlag;
+}
+
 
 static void LanguageRTF(TconvertedPass1_RTF *cq)
 {
@@ -1372,14 +1467,14 @@ FILE *ImgFile;
 string FileName;
 TBox Box;
 
-  initBox(Box);  
-  Box.HorizontalPos = 3;	/*3-Full */  
+  initBox(Box);
+  Box.HorizontalPos = 3;	/*3-Full */
   //Box.Image_type = 0;		/*external*/
-  Box.Contents = 3; 		/*contents image*/  
+  Box.Contents = 3; 		/*contents image*/
 
   while (!feof(cq->wpd))
 	{
-	cq->subby = RTFLoadKeyword(cq);
+	cq->subby = cq->RTFLoadKeyword();
 	cq->ProcessKeyRTF();
 
 //	if(cq->subby>=202 && cq->subby<=204) printf("%s\n",cq->Keyword() );
@@ -1388,11 +1483,12 @@ TBox Box;
 
 	if(cq->subby==205)	//regular char
 	   {
+	   //if(cq->log) fprintf(cq->log," pos=%Xh ",ftell(cq->wpd));
 	   if(isxdigit(cq->by))
 		{
 		ungetc(cq->by,cq->wpd);
 
-                FileName = MergePaths(OutputDir,RelativeFigDir)+GetSomeImgName();
+		FileName = MergePaths(OutputDir,RelativeFigDir)+GetSomeImgName();
 		ImgFile = OpenWrChk(FileName(),"wb",cq->err);
 		if(ImgFile == NULL) break;
 
@@ -1402,7 +1498,7 @@ TBox Box;
 		ImageWP(cq,FileName,Box);
 #ifndef DEBUG
 		//printf("\n%d %s\n",i,FileName());
-                if(SaveWPG<0)
+		if(SaveWPG<0)
 		    unlink(FileName());
 #endif
 		break;
@@ -1528,8 +1624,8 @@ char c;
 	}
 
 /*  if(Type=="blipuid")
-	{
-	}*/
+  {
+  } */
 
   if(Type=="objdata")
 	{
@@ -1612,7 +1708,7 @@ int KeywordType;
 
   if (cGroup < 0) return ecStackUnderflow;
 
-  if(by==0) KeywordType=RTFLoadKeyword(this);
+  if(by==0) KeywordType=RTFLoadKeyword();
 	   else KeywordType=subby;
 
   switch(KeywordType)
@@ -1735,15 +1831,15 @@ int KeywordType;
 		  }
 	      if(Keyword=="footer")
 		  {
-		  HeaderFooterRTF(this,6); break;		// 0110b
+		  HeaderFooterRTF(6); break;		// 0110b
 		  }
 	      if(Keyword=="footerl")
 		  {
-		  HeaderFooterRTF(this,0xE); break;	// 1110b
+		  HeaderFooterRTF(0xE); break;	// 1110b
 		  }
 	      if(Keyword=="footerr")
 		  {
-		  HeaderFooterRTF(this,0xA); break;	// 1010b
+		  HeaderFooterRTF(0xA); break;	// 1010b
 		  }
 	      if(Keyword=="footnote")
 		  {
@@ -1757,15 +1853,15 @@ int KeywordType;
 		  }
 	      if(Keyword=="header")
 		  {
-		  HeaderFooterRTF(this,4);break;		// 0100b
+		  HeaderFooterRTF(4);break;		// 0100b
 		  }
 	      if(Keyword=="headerl")
 		  {
-		  HeaderFooterRTF(this,0xC);break;	// 1100b
+		  HeaderFooterRTF(0xC);break;	// 1100b
 		  }
 	      if(Keyword=="headerr")
 		  {
-		  HeaderFooterRTF(this,8);break;		// 1000b
+		  HeaderFooterRTF(8);break;		// 1000b
 		  }
 	      if(Keyword=="highlight")
 		  {
@@ -1961,22 +2057,39 @@ int KeywordType;
 		  break;
 		  }
 	      if(Keyword=="uldb")
-		  {
-		  if(!isParam) lParam=1;
-		  if(lParam) AttrOn(attr,11);	/* Start double underline */
-			    else AttrOff(this,11);	/* End double underline */
-		  break;
-		  }
+              {
+		if(!isParam) lParam=1;
+		if(lParam) AttrOn(attr,11);	/* Start double underline */
+		      else AttrOff(this,11);	/* End double underline */
+	        break;
+	      }
 	      if(Keyword=="ulnone")
-		  {
-		  AttrOff(this,14); AttrOff(this,11);	/* End of all underlines */
-		  break;
-		  }
+	      {
+		AttrOff(this,14); AttrOff(this,11);	/* End of all underlines */
+		break;
+	      }
 	      if(Keyword=="xe")
-		  {
-		  IndexRTF(this);
-		  break;
-		  }
+	      {
+	        IndexRTF();
+	        break;
+	      }
+
+	      if(Keyword=="trowd")	// Mandatory row definition for every table
+	      {
+                ScanTableFormat();
+	        break;
+	      }
+	      if(Keyword=="cell")
+              {
+                if(char_on_line==0) char_on_line=-1;
+	        CellTable(this);
+                break;
+              }
+	      if(Keyword=="row")
+              {
+		RowTable(this);
+                break;
+              }
 
 	      *(ObjType)='!';
 	      ecTranslateKeyword(Keyword(), lParam, isParam); //keyword+arg
