@@ -2,7 +2,7 @@
  * program:     wp2latex                                                      *
  * function:    module for conversion SVG files into LaTeX		      *
  * modul:       pass1svg.cc                                                   *
- * description: This module contains parser for SVG documents. It could   *
+ * description: This module contains parser for SVG documents. It could       *
  *		be optionally compiled with WP2LaTeX package.		      *
  * licency:     GPL		                                              *
  ******************************************************************************/
@@ -83,7 +83,12 @@ bool ReadFloat(const char **pstr, float &f)
   f = atof(str);
 
   if(*str=='-') str++;
-  while(isdigit(*str) || *str=='.') str++;
+  while(isdigit(*str)) str++;
+  if(*str=='.')		// Only one dot is tolerated inside number.
+  {
+    str++;
+    while(isdigit(*str)) str++;
+  }
 
   while(isspace(*str)) str++;
 
@@ -169,8 +174,13 @@ protected:
      ForwardHolder **DelayedObjects;
      int DelayedObjCount;
 
+     //RGB_Record DefaultLineColor;
+     //RGB_Record DefaultFillColor;
+     //unsigned char DefaultLineStyle;
+     //unsigned char DefaultBrushStyle;
+
      bool GetProp(const char *PropName, float &f);
-     bool GetColor(RGB_Record &LineColor, const char *ClrType=LINE_COLOR);
+     int GetColor(RGB_Record &LineColor, const char *ClrType=LINE_COLOR);
      void GetLineCap(unsigned char &LineCap);
      void GetLineJoin(unsigned char &LineJoin);
      void GetStyle(vecPen *pPen, vecBrush *pBrush);
@@ -181,7 +191,7 @@ protected:
      void ClosePathCurve(float **pPoints, int & n, bool Close=false, AbstractTransformXY *Tx=NULL);
 
 public:
-     TconvertedPass1_SVG(void)	{DelayedObjCount=DObjectCount=0;objs=NULL;DelayedObjects=NULL;}
+     TconvertedPass1_SVG(void);
      virtual ~TconvertedPass1_SVG(void) {EraseObjCache();}
 
      void AddDefine(DefinedObject *pObj);
@@ -224,7 +234,15 @@ ImplementReader(SVG,".SVG")
    LoadPictureSVG,
    NULL);
 
-#define SVGVersion "0.7"
+#define SVGVersion "0.9"
+
+
+TconvertedPass1_SVG::TconvertedPass1_SVG(void): DelayedObjects(NULL), objs(NULL) 
+{
+  PSS.LineStyle = 0;
+  PSS.FillPattern = FILL_NONE;
+  DelayedObjCount = DObjectCount = 0;
+}
 
 
 void TconvertedPass1_SVG::EraseObjCache(void)
@@ -360,14 +378,18 @@ static SVG_color SVG_colors[] =
 };
 
 
-bool TconvertedPass1_SVG::GetColor(RGB_Record &LineColor, const char *ClrType)
+/** Decode color.
+ * @return -1 on failure; 0 when no color specified; 1 Color is OK */
+int TconvertedPass1_SVG::GetColor(RGB_Record &LineColor, const char *ClrType)
 {
 int i;
 
-  if(ClrType==NULL || *ClrType==0) return false;
-  if((i=TAG_Args IN ClrType)<0) return false;
+  if(ClrType==NULL || *ClrType==0) return -1;
+  if((i=TAG_Args IN ClrType)<0) return -1;
   const char * const Payload = TAG_Args.Member(i,1);
-  if(Payload==NULL) return false;
+  if(Payload==NULL) return -1;
+
+  if(!strcmp(Payload, "none")) return 0;
 
   if(Payload[0]=='#' && isxdigit(Payload[1]))
   {
@@ -375,16 +397,16 @@ int i;
     LineColor.Blue = val & 0xFF;
     LineColor.Green = (val>>8) & 0xFF;
     LineColor.Red = (val>>16) & 0xFF;
-    return true;
+    return 1;
   }
   for(i=0; i<sizeof(SVG_colors)/sizeof(SVG_color);i++)
     if(!strcmp(Payload, SVG_colors[i].desc))
     {
       LineColor = SVG_colors[i].color;
-      return true;
+      return 1;
     }
 
-return false;
+return -1;
 }
 
 
@@ -518,6 +540,20 @@ const char *str;
 void TconvertedPass1_SVG::GetStyle(vecPen *pPen, vecBrush *pBrush)
 {
 int i;
+
+/*
+  if(pPen)
+  {
+    pPen->LineStyle = DefaultLineStyle;	// Default LineStyle is none.
+    memcpy(&pPen->LineColor, &DefaultLineColor, sizeof(DefaultLineColor));
+  }
+  if(pBrush)
+  {
+    pBrush->BrushStyle = DefaultBrushStyle;	// Default LineStyle is none.
+    memcpy(&pBrush->FillColor, &DefaultFillColor, sizeof(DefaultFillColor));
+  }
+*/
+
   if((i=TAG_Args IN "style")>=0)
     GetStyle(pPen, pBrush, TAG_Args.Member(i,1));
 
@@ -550,6 +586,12 @@ int i;
       do
         { i++;}
       while(isdigit(StrPoints[i]));
+      if(StrPoints[i]=='.')
+      {
+        do
+        { i++;}
+        while(isdigit(StrPoints[i]));
+      }
       continue;
     }
     i++;
@@ -568,6 +610,12 @@ int i;
       do
         { i++;}
       while(isdigit(StrPoints[i]));
+      if(StrPoints[i]=='.')
+      {
+        do
+        { i++;}
+        while(isdigit(StrPoints[i]));
+      }
       continue;
     }
     i++;
@@ -629,13 +677,14 @@ int i;
       if(ReadFloat(&StrTransform, Tx(3,0)))
       {
         Tx(0,0) = Tx(1,1) = Tx(2,2) = Tx(3,3) = 1;
-        if(*StrTransform==',')
+        if(*StrTransform==',') 
         {
           StrTransform++;
-          if(ReadFloat(&StrTransform, Tx(3,1)))
-          {
-            pTransformSVG->CTM *= Tx;
-          }
+          while(isspace(*StrTransform)) StrTransform++;
+        }
+        if(ReadFloat(&StrTransform, Tx(3,1)))
+        {
+          pTransformSVG->CTM *= Tx;
         }
       }
     } else if(!strncmp("scaleX",StrTransform,6))
@@ -673,13 +722,14 @@ int i;
         if(ReadFloat(&StrTransform, Tx(0,0)))
         {
            Tx(2,2) = Tx(3,3) = 1;
-           if(*StrTransform==',')
+           if(*StrTransform==',') 
            {
              StrTransform++;
-             if(ReadFloat(&StrTransform, Tx(1,1)))
-             {
-               pTransformSVG->CTM *= Tx;
-             }
+             while(isspace(*StrTransform)) StrTransform++;
+           }
+           if(ReadFloat(&StrTransform, Tx(1,1)))
+           {
+             pTransformSVG->CTM *= Tx;
            }
         }
       }
@@ -807,6 +857,14 @@ float stroke_width;
     GetColor(pVectCirc->LineColor);
     GetLineCap(pVectCirc->LineCap);
     GetLineJoin(pVectCirc->LineJoin);
+    switch(GetColor(pVectCirc->FillColor,FILL_COLOR))
+    {
+      case 0: pVectCirc->BrushStyle=FILL_NONE;
+              break;
+      case 1: if(pVectCirc->BrushStyle==FILL_NONE) pVectCirc->BrushStyle=FILL_SOLID;
+              break;    
+      //default:
+    }
 
     UpdateBBox(bbx, 0, cx-r-pVectCirc->PenWidth/2, cy-r-pVectCirc->PenWidth/2, 2*r+pVectCirc->PenWidth, 2*r+pVectCirc->PenWidth);
     VectList.AddObject(pVectCirc);
@@ -1129,7 +1187,7 @@ RGB_Record FillColor;
   }
 
   VectorLine *pVecLine;
-  if(GetColor(FillColor,FILL_COLOR))
+  if(GetColor(FillColor,FILL_COLOR) > 0)
   {
      VectorPolygon *pVecPoly = new VectorPolygon(*pPoints,n);
      //pVecPoly->AttribFromPSS(PSS);
@@ -1199,10 +1257,14 @@ float stroke_width;
   GetColor(pVecCurve->LineColor);
   GetLineCap(pVecCurve->LineCap);
   GetLineJoin(pVecCurve->LineJoin);
-  if(GetColor(pVecCurve->FillColor,FILL_COLOR))
+  switch(GetColor(pVecCurve->FillColor,FILL_COLOR))
   {
-    if(pVecCurve->BrushStyle==FILL_NONE) pVecCurve->BrushStyle=FILL_SOLID;
-    //pVecCurve->Filled = true;
+    case 0: pVecCurve->BrushStyle=FILL_NONE;
+            break;
+    case 1: if(pVecCurve->BrushStyle==FILL_NONE) pVecCurve->BrushStyle=FILL_SOLID;
+	    //pVecCurve->Filled = true;
+            break;    
+    //default:
   }
 
   pVecCurve->Close = Close;
@@ -1345,6 +1407,12 @@ return;
 }
 
 
+float SolveQuadraticPos(const float anch, const float s)
+{
+  return (s-anch) / sqrt(2.0) + anch;
+}
+
+
 void TconvertedPass1_SVG::Path(void)
 {
 #ifdef DEBUG
@@ -1375,9 +1443,14 @@ VectorList VectListIn;
   VectListIn.Swap(VectList);
   while(*StrMovements != 0)
   {
-    while(isspace(*StrMovements)) StrMovements++;
-    if(*StrMovements==0) break;
-    switch((isdigit(*StrMovements) || *StrMovements=='.' || *StrMovements=='-') ? LastCommand : *StrMovements++)
+    char ch = *StrMovements;
+    if(ch==0) break;
+    if(isspace(ch)) {StrMovements++;continue;}
+    if(ch==',') {StrMovements++;continue;}	// Need to gobble whitespaces after comma.
+
+    if(!(isdigit(ch) || ch=='.' || ch=='-'))
+        LastCommand = *StrMovements++;
+    switch(LastCommand)
     {
       case 'M':			//move to.
         if(n>0) SegmentCount++;
@@ -1456,9 +1529,11 @@ VectorList VectListIn;
           if(Points==NULL) goto ExitErr;
         }
 
-        ReadFloat(&StrMovements, Points[2*n-2]);
-        Points[2*n-2] += PositionOffsetX;
-        Points[2*n-1] = PositionY + PositionOffsetY;
+        if(ReadFloat(&StrMovements, PositionX))
+        {
+          Points[2*n-2] = PositionX + PositionOffsetX;
+          Points[2*n-1] = PositionY + PositionOffsetY;
+        }
         break;
 
      case 'h':		// horizontal relative line to.
@@ -1488,7 +1563,7 @@ VectorList VectListIn;
         PositionX += val;
         Points[2*n-2] = PositionX;
         Points[2*n-1] = PositionY;
-        break;
+	break;
 
      case 'V':		//vertical line to.
         LastCommand = 'V';
@@ -1513,9 +1588,11 @@ VectorList VectListIn;
             if(Points==NULL) goto ExitErr;
           }
 
-          ReadFloat(&StrMovements, Points[2*n-1]);
-          Points[2*n-1] += PositionOffsetY;
-          Points[2*n-2] = PositionX + PositionOffsetX;
+          if(ReadFloat(&StrMovements, PositionY))
+          {
+            Points[2*n-1] = PositionY + PositionOffsetY;
+            Points[2*n-2] = PositionX + PositionOffsetX;
+          }
         break;
 
      case 'v':		// vertical relative line to.
@@ -1548,13 +1625,13 @@ VectorList VectListIn;
         break;
 
      case 'L':		//line to.
-        LastCommand = 'L';
-        if(isCurve)
-        {
-          if(n>0) SegmentCount++;
-          ClosePathCurve(&Points,n,false,Tx);
-          isCurve = false;
-        }
+          LastCommand = 'L';
+          if(isCurve)
+          {
+            if(n>0) SegmentCount++;
+            ClosePathCurve(&Points,n,false,Tx);
+            isCurve = false;
+          }
 
           if(Points==NULL)
           {
@@ -1578,10 +1655,12 @@ VectorList VectListIn;
             ReadFloat(&StrMovements, Points[2*n-1]);
           }
           else
-            Points[2*n-1] = PositionY;
+            Points[2*n-1] = PositionY - PositionOffsetY;
 
           Points[2*n-2] += PositionOffsetX;
           Points[2*n-1] += PositionOffsetY;
+          PositionX = Points[2*n-2];
+          PositionY = Points[2*n-1];
        break;
 
      case 'l':		//relative line to.
@@ -1642,24 +1721,127 @@ VectorList VectListIn;
             if(Points==NULL) goto ExitErr;
           }
 
-          ReadFloat(&StrMovements, Points[2+6*n-6]);
-          if(*StrMovements==',') StrMovements++;
-          ReadFloat(&StrMovements, Points[2+6*n-5]);
-          if(*StrMovements==',') StrMovements++;
-          ReadFloat(&StrMovements, Points[2+6*n-4]);
-          if(*StrMovements==',') StrMovements++;
-          ReadFloat(&StrMovements, Points[2+6*n-3]);
-          if(*StrMovements==',') StrMovements++;
-          ReadFloat(&StrMovements, Points[2+6*n-2]);
-          if(*StrMovements==',') StrMovements++;
-          ReadFloat(&StrMovements, Points[2+6*n-1]);
+          {
+            float * const PointBase = Points + 2 + 6*n;
+            ReadFloat(&StrMovements, PointBase[-6]); // x1
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, PointBase[-5]); // y1 - the control point at beginning of curve.
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, PointBase[-4]); // x2
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, PointBase[-3]); // y2 - the control point at end of curve.
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, PointBase[-2]); // x
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, PointBase[-1]); // y - the end point for curve.
 
-          Points[2*n-6] += PositionOffsetX;
-          Points[2*n-4] += PositionOffsetX;
-          Points[2*n-2] += PositionOffsetX;
-          Points[2*n-5] += PositionOffsetY;
-          Points[2*n-3] += PositionOffsetY;
-          Points[2*n-1] += PositionOffsetY;
+            PointBase[-6] += PositionOffsetX;
+            PointBase[-4] += PositionOffsetX;
+            PointBase[-2] += PositionOffsetX;
+            PointBase[-5] += PositionOffsetY;
+            PointBase[-3] += PositionOffsetY;
+            PointBase[-1] += PositionOffsetY;
+         }
+        break;
+
+    case 'Q':		//quadratic curve
+        LastCommand = 'Q';
+        if(!isCurve)
+        {
+          if(n>0) SegmentCount++;
+          ClosePathLine(&Points,n,false,Tx);
+          isCurve = true;
+        }
+
+          if(Points==NULL)
+          {
+            Points=(float*)malloc(8*sizeof(float));
+            if(Points==NULL) goto ExitErr;
+            Points[0]=PositionX; Points[1]=PositionY;
+            n = 1;
+          }
+          else
+          {
+            n++;
+            Points = (float*)realloc(Points,(2+6*n)*sizeof(float));
+            if(Points==NULL) goto ExitErr;
+          }
+
+          {
+            float * const PointBase = Points + 2 + 6*n;
+            ReadFloat(&StrMovements, PointBase[-6]); // xt
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, PointBase[-5]); // yt
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, PointBase[-2]); // x
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, PointBase[-1]); // y
+          
+            PointBase[-4] = SolveQuadraticPos(PointBase[-2], PointBase[-6]);
+            PointBase[-3] = SolveQuadraticPos(PointBase[-1], PointBase[-5]);
+            PointBase[-6] = SolveQuadraticPos(PointBase[-8], PointBase[-6]);
+            PointBase[-5] = SolveQuadraticPos(PointBase[-7], PointBase[-5]);
+
+            PointBase[-6] += PositionOffsetX;
+            PointBase[-4] += PositionOffsetX;
+            PointBase[-2] += PositionOffsetX;
+            PointBase[-5] += PositionOffsetY;
+            PointBase[-3] += PositionOffsetY;
+            PointBase[-1] += PositionOffsetY;
+          }
+        break;
+
+    case 'T':		//quadratic short curve
+        LastCommand = 'T';
+        if(!isCurve)
+        {
+          if(n>0) SegmentCount++;
+          ClosePathLine(&Points,n,false,Tx);
+          isCurve = true;
+        }
+
+        if(Points==NULL)
+        {
+          Points=(float*)malloc(8*sizeof(float));
+          if(Points==NULL) goto ExitErr;
+          Points[0]=PositionX; Points[1]=PositionY;
+          n = 1;
+        }
+        else
+        {
+          n++;
+          Points = (float*)realloc(Points,(2+6*n)*sizeof(float));
+          if(Points==NULL) goto ExitErr;
+        }
+
+        {
+          float * const PointBase = Points + 2 + 6*n;
+          ReadFloat(&StrMovements, PointBase[-2]); // x
+          if(*StrMovements==',') StrMovements++;
+          ReadFloat(&StrMovements, PointBase[-1]); // y
+
+          if(n<=1)
+          {
+            PointBase[-6] = PointBase[-4] = (PointBase[-8]+PointBase[-2])/2;
+            PointBase[-5] = PointBase[-3] = (PointBase[-7]+PointBase[-1])/2;
+          }
+          else
+          {
+            val = (PointBase[-8]-PointBase[-10])*(1+sqrt(2.0)) + PointBase[-10];
+            PointBase[-6] = SolveQuadraticPos(PointBase[-8], val);
+            PointBase[-4] = SolveQuadraticPos(PointBase[-2], val);
+            val = (PointBase[-7]-PointBase[-9])*(1+sqrt(2.0)) + PointBase[-9];
+            PointBase[-5] = SolveQuadraticPos(PointBase[-7], val); 
+            PointBase[-3] = SolveQuadraticPos(PointBase[-1], val);
+          }
+
+          PointBase[-6] += PositionOffsetX;
+          PointBase[-4] += PositionOffsetX;
+          PointBase[-2] += PositionOffsetX;
+          PointBase[-5] += PositionOffsetY;
+          PointBase[-3] += PositionOffsetY;
+          PointBase[-1] += PositionOffsetY;
+        }
         break;
 
     case 'c':		//relative curve to.
@@ -1685,26 +1867,78 @@ VectorList VectListIn;
             if(Points==NULL) goto ExitErr;
           }
 
-          ReadFloat(&StrMovements, val);
-          Points[2+6*n-6] = PositionX + val;
-          if(*StrMovements==',') StrMovements++;
-          ReadFloat(&StrMovements, val);
-          Points[2+6*n-5] = PositionY + val;
-          if(*StrMovements==',') StrMovements++;
-          ReadFloat(&StrMovements, val);
-          Points[2+6*n-4] = PositionX + val;
-          if(*StrMovements==',') StrMovements++;
-          ReadFloat(&StrMovements, val);
-          Points[2+6*n-3] = PositionY + val;
+         {
+            float * const PointBase = Points + 2 + 6*n;
 
-          if(*StrMovements==',') StrMovements++;
-          ReadFloat(&StrMovements, val);
-          PositionX += val;
-          Points[2+6*n-2] = PositionX;
-          if(*StrMovements==',') StrMovements++;
-          ReadFloat(&StrMovements, val);
-          PositionY += val;
-          Points[2+6*n-1] = PositionY;
+            ReadFloat(&StrMovements, val);
+            PointBase[-6] = PositionX + val;
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, val);
+            PointBase[-5] = PositionY + val;
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, val);
+            PointBase[-4] = PositionX + val;
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, val);
+            PointBase[-3] = PositionY + val;
+
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, val);
+            PositionX += val;
+            PointBase[-2] = PositionX;
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, val);
+            PositionY += val;
+            PointBase[-1] = PositionY;
+          }
+        break;
+
+    case 'q':		//relative quadratic curve to.
+        LastCommand = 'q';
+        if(!isCurve)
+        {
+          if(n>0) SegmentCount++;
+          ClosePathLine(&Points,n,false,Tx);
+          isCurve = true;
+        }
+
+          if(Points==NULL)
+          {
+            Points=(float*)malloc(8*sizeof(float));
+            if(Points==NULL) goto ExitErr;
+            Points[0]=PositionX; Points[1]=PositionY;
+            n = 1;
+          }
+          else
+          {
+            n++;
+            Points = (float*)realloc(Points,(2+6*n)*sizeof(float));
+            if(Points==NULL) goto ExitErr;
+          }
+
+          {
+            float * const PointBase = Points + 2 + 6*n;
+
+            ReadFloat(&StrMovements, val);
+            PointBase[-6] = PositionX + val;
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, val);
+            PointBase[-5] = PositionY + val;
+            if(*StrMovements==',') StrMovements++;
+
+            ReadFloat(&StrMovements, val);
+            PositionX += val;
+            PointBase[-2] = PositionX;
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, val);
+            PositionY += val;
+            PointBase[-1] = PositionY;
+            
+            PointBase[-4] = SolveQuadraticPos(PointBase[-2], PointBase[-6]);
+            PointBase[-3] = SolveQuadraticPos(PointBase[-1], PointBase[-5]);
+            PointBase[-6] = SolveQuadraticPos(PointBase[-8], PointBase[-6]);
+            PointBase[-5] = SolveQuadraticPos(PointBase[-7], PointBase[-5]);
+          }
         break;
 
     case 'S':		// shorter curve to.
@@ -1729,21 +1963,24 @@ VectorList VectListIn;
           if(Points==NULL) goto ExitErr;
         }
 
-        ReadFloat(&StrMovements, Points[2+6*n-4]);
-        if(*StrMovements==',') StrMovements++;
-        ReadFloat(&StrMovements, Points[2+6*n-3]);
-        ReadFloat(&StrMovements, Points[2+6*n-2]);
-        if(*StrMovements==',') StrMovements++;
-        ReadFloat(&StrMovements, Points[2+6*n-1]);
-        if(n==1)
         {
-          Points[2+6*n-6] = Points[2+6*n-4];
-          Points[2+6*n-5] = Points[2+6*n-3];
-        }
-        else
-        {
-          Points[2+6*n-6] = 2*Points[2+6*n-8] - Points[2+6*n-10];
-          Points[2+6*n-5] = 2*Points[2+6*n-7] - Points[2+6*n-9];
+          float * const PointBase = Points + 2 + 6*n;
+          ReadFloat(&StrMovements, PointBase[-4]);
+          if(*StrMovements==',') StrMovements++;
+          ReadFloat(&StrMovements, PointBase[-3]);
+          ReadFloat(&StrMovements, PointBase[-2]);
+          if(*StrMovements==',') StrMovements++;
+          ReadFloat(&StrMovements, PointBase[-1]);
+          if(n==1)
+          {
+            PointBase[-6] = PointBase[-4];
+            PointBase[-5] = PointBase[-3];
+          }
+          else
+          {
+            PointBase[-6] = 2*PointBase[-8] - PointBase[-10];
+            PointBase[-5] = 2*PointBase[-7] - PointBase[-9];
+          }
         }
         break;
 
@@ -1770,28 +2007,79 @@ VectorList VectListIn;
             if(Points==NULL) goto ExitErr;
           }
 
-          ReadFloat(&StrMovements, Points[2+6*n-4]);
-          Points[2+6*n-4] += PositionX;
-          if(*StrMovements==',') StrMovements++;
-          ReadFloat(&StrMovements, Points[2+6*n-3]);
-          Points[2+6*n-3] += PositionY;
-          ReadFloat(&StrMovements, Points[2+6*n-2]);
-          Points[2+6*n-2] += PositionX;
-          if(*StrMovements==',') StrMovements++;
-          ReadFloat(&StrMovements, Points[2+6*n-1]);
-          Points[2+6*n-1] += PositionY;
-          if(n==1)
           {
-            Points[2+6*n-6] = Points[2+6*n-4];
-            Points[2+6*n-5] = Points[2+6*n-3];
+            float * const PointBase = Points + 2 + 6*n;
+            ReadFloat(&StrMovements, PointBase[-4]);
+            PointBase[-4] += PositionX;
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, PointBase[-3]);
+            PointBase[-3] += PositionY;
+            ReadFloat(&StrMovements, PointBase[-2]);
+            PointBase[-2] += PositionX;
+            if(*StrMovements==',') StrMovements++;
+            ReadFloat(&StrMovements, PointBase[-1]);
+            PointBase[-1] += PositionY;
+            if(n==1)
+            {
+              PointBase[-6] = PointBase[-4];
+              PointBase[-5] = PointBase[-3];
+            }
+            else
+            {
+              PointBase[-6] = 2*PointBase[-8] - PointBase[-10];
+              PointBase[-5] = 2*PointBase[-7] - PointBase[-9];
+            }
+            PositionX = PointBase[-2];
+            PositionY = PointBase[-1];
+          }
+        break;
+
+    case 't':		// relative shorter quadratic curve to
+        LastCommand = 't';
+        if(!isCurve)
+        {
+          if(n>0) SegmentCount++;
+          ClosePathLine(&Points,n,false,Tx);
+          isCurve = true;
+        }
+
+        if(Points==NULL)
+        {
+          Points=(float*)malloc(8*sizeof(float));
+          if(Points==NULL) goto ExitErr;
+          Points[0]=PositionX; Points[1]=PositionY;
+          n = 1;
+        }
+        else
+        {
+          n++;
+          Points = (float*)realloc(Points,(2+6*n)*sizeof(float));
+          if(Points==NULL) goto ExitErr;
+        }
+
+        {
+          float * const PointBase = Points + 2 + 6*n;
+          ReadFloat(&StrMovements, val);
+          PointBase[-2] = PositionX = PositionX + val;
+          if(*StrMovements==',') StrMovements++;
+          ReadFloat(&StrMovements, val);
+          PointBase[-1] = PositionY = PositionY + val;
+
+          if(n<=1)
+          {
+            PointBase[-6] = PointBase[-4] = (PointBase[-8]+PointBase[-2])/2;
+            PointBase[-5] = PointBase[-3] = (PointBase[-7]+PointBase[-1])/2;
           }
           else
           {
-            Points[2+6*n-6] = 2*Points[2+6*n-8] - Points[2+6*n-10];
-            Points[2+6*n-5] = 2*Points[2+6*n-7] - Points[2+6*n-9];
+            val = (PointBase[-8]-PointBase[-10])*(1+sqrt(2.0)) + PointBase[-10];
+            PointBase[-6] = SolveQuadraticPos(PointBase[-8], val);
+            PointBase[-4] = SolveQuadraticPos(PointBase[-2], val);
+            val = (PointBase[-7]-PointBase[-9])*(1+sqrt(2.0)) + PointBase[-9];
+            PointBase[-5] = SolveQuadraticPos(PointBase[-7], val); 
+            PointBase[-3] = SolveQuadraticPos(PointBase[-1], val);
           }
-          PositionX = Points[2+6*n-2];
-          PositionY = Points[2+6*n-1];
+        }
         break;
 
       case 'Z':		// close path
@@ -1811,15 +2099,45 @@ VectorList VectListIn;
           ClosePathLine(&Points, n, SegmentCount<=0, Tx);
         }
         SegmentCount = 0;
+		// Z command breaks PATH.
+        if(VectList.FirstObject != NULL)
+        {
+          VectorPath *pVectPath = new VectorPath;
+          if(pVectPath)
+          {
+            GetStyle(pVectPath,pVectPath);
+            float stroke_width;
+            if(GetProp("stroke-width",stroke_width))
+            {
+              pVectPath->PenWidth = stroke_width;
+            }
+            switch(GetColor(pVectPath->FillColor,FILL_COLOR))
+            {
+              case 0: pVectPath->BrushStyle=FILL_NONE;
+                      break;
+              case 1: if(pVectPath->BrushStyle==FILL_NONE) pVectPath->BrushStyle=FILL_SOLID;
+	              //pVectPath->Filled = true;
+                      break;    
+              //default:
+            }
+            GetColor(pVectPath->LineColor);
+            GetLineCap(pVectPath->LineCap);
+            GetLineJoin(pVectPath->LineJoin);
+            //pVectPath->Close = Close;
+
+            pVectPath->Swap(VectList);
+            VectListIn.AddObject(pVectPath);
+          }
+        }
         break;
 
       default:
-        if(isdigit(*StrMovements) || *StrMovements=='.' || *StrMovements=='-')
-            StrMovements++;		// Not incremented in switch argument.
+        if(isdigit(ch) || ch=='.' || ch=='-')
+            StrMovements++;
         else
         {
           if(err!=NULL)
-              fprintf(err,_("\nError: Unsupported command '%c' in SVG path!"),*(StrMovements-1));
+              fprintf(err,_("\nError: Unsupported path command '%c' in SVG path!"),ch);
         }
         break;
 
@@ -1844,10 +2162,14 @@ VectorList VectListIn;
       {
         pVectPath->PenWidth = stroke_width;
       }
-      if(GetColor(pVectPath->FillColor,FILL_COLOR))
+      switch(GetColor(pVectPath->FillColor,FILL_COLOR))
       {
-        if(pVectPath->BrushStyle==FILL_NONE) pVectPath->BrushStyle=FILL_SOLID;
-        //pVectPath->Filled = true;
+        case 0: pVectPath->BrushStyle=FILL_NONE;
+                break;
+        case 1: if(pVectPath->BrushStyle==FILL_NONE) pVectPath->BrushStyle=FILL_SOLID;
+                //pVectPath->Filled = true;
+                break;    
+        //default:
       }
       GetColor(pVectPath->LineColor);
       GetLineCap(pVectPath->LineCap);
@@ -1861,6 +2183,12 @@ VectorList VectListIn;
 
   strcpy(ObjType,ObjName+1);
   if(Tx) delete Tx;
+/*
+#ifdef _DEBUG
+  FILE*F=fopen("O:\\temp\\38\\dump.txt","wb");
+  if(F) {VectList.Dump2File(F);fclose(F);}
+#endif
+*/
   return;
 
 ExitErr:
@@ -2318,20 +2646,42 @@ int RetVal = 0;
   ConvertHTML = GetTranslator("htmlTOinternal");
   CharReader = &ch_fgetc;
 
-  TablePos=0;
+  TablePos = 0;
 
   ActualPos = DocumentStart = ftell(wpd);
   fsize = FileSize(wpd);
   perc.Init(ftell(wpd), fsize,_("First pass SVG:") );
 
   while(ActualPos < fsize)
+  {
+    ReadXMLTag(false);
+    if(TAG=="<SVG>" || TAG=="<svg>")
+    {
+      if(GetColor(PSS.LineColor) > 0)
       {
-      if(Verbosing >= 1)		//actualise a procentage counter
-	      perc.Actualise(ActualPos);
-
-      TAG.erase();
-      ProcessKeySVG();
+        PSS.LineStyle = 1;
       }
+      switch(GetColor(PSS.FillColor,FILL_COLOR))
+      {
+        case 0: PSS.FillPattern=FILL_NONE;
+                break;
+        case 1: if(PSS.FillPattern==FILL_NONE) PSS.FillPattern=FILL_SOLID;
+	        //pVecCurve->Filled = true;
+                break;    
+        //default:
+      }
+      break;
+    }
+  }  
+
+  while(ActualPos < fsize)
+  {
+    if(Verbosing >= 1)		//actualise a procentage counter
+	perc.Actualise(ActualPos);
+
+    TAG.erase();
+    ProcessKeySVG();
+  }
   SolveDelayedObjs();
 
   ProcessImage();
@@ -2390,7 +2740,7 @@ size_t fsize;
       ConvSvg.ConvertHTML = GetTranslator("htmlTOinternal");
       ConvSvg.CharReader = &ch_fgetc;
 
-      ConvSvg.TablePos=0;
+      ConvSvg.TablePos = 0;
       ConvSvg.ActualPos = ConvSvg.DocumentStart = ftell(ConvSvg.wpd);
       fsize = FileSize(ConvSvg.wpd);
       //ConvSvg.perc.Init(ftell(ConvSvg.wpd), fsize,_("First pass SVG:") );
