@@ -72,15 +72,15 @@ extern int UseMemStream;
 /// Definition of methods of the Percentor class - only integer arithmetic due to the speed problems.
 void Percentor::Actualise(long NewValue)
 {
-  NewValue-=StartValue;
-  NewValue>>=Shift;
-  NewValue = (100*NewValue)/Range;
+ NewValue -= StartValue;
+ NewValue >>= Shift;
+ NewValue = (100*NewValue)/Range;
 
-  if(NewValue!=CurrentPercent)
-	{
-	CurrentPercent=NewValue;
-	Show();
-	}		
+ if(NewValue != CurrentPercent)
+ {
+   CurrentPercent = NewValue;
+   Show();
+ }		
 }
 
 
@@ -145,7 +145,7 @@ void Percentor::Show(void)
 }
 
 
-/** This function reads string from standard input. */
+/** This function reads string from standard console input. */
 void StrGets(string & s)
 {
 char c;
@@ -220,9 +220,9 @@ void RunError(int ExitCode, const char *s, const void *arg)
     case 0x15: fprintf(err,_("Info: Please report this situation to enable further developping WP2LaTeX.\n%s"),chk(s));
 	       break;
     case 0x20: fprintf(err,_("Error: Output device is full.\n"));
-    	       break;           
+    	       break;
     case 0x8B: fprintf(err,_("Fatal: Program Error. Write full bug report.\n"));
-               break;           
+               break;
     case 0x200:fprintf(err,_("Error: No filename entered!\n"));
                break;
     case 0x201:fprintf(err,_("Error: File \"%s\" is not a WordPerfect document!\n"),wpd_filename());
@@ -321,7 +321,7 @@ static bool printed=false;
 
  if(Verbosing < 1) return;
  printf(_("\n>>>WP2LaTeX<<< Conversion program: From Wordperfect to LaTeX Version %s \n"
-	  "  (c) Made by J.Fojtik  ---- Date : 1996-2023\n"),
+	  "  (c) Made by J.Fojtik  ---- Date : 1996-2024\n"),
 	    VersionWP2L);
 }
 
@@ -338,7 +338,7 @@ return '*';
 /** This function prints wp2latex help to the text console. */
 void ShowHelp(void)
 {
- puts(_("\nWrite any suggestions, reports or bugs to address FOJTIK@PENGUIN.CZ"
+ puts(_("\nWrite any suggestions, reports or bugs to address JaFojtik@seznam.cz"
 	  "\nor JaFojtik@yandex.com."));
 
  puts(_("\nPackage signalling: '-' disabled, '+' enable on demand, '*' enabled always."));
@@ -404,7 +404,7 @@ void ShowHelp(void)
 #endif
 	  ExtendedCheck==true?"*":"",
 	  Verbosing>0?"":"*",
-	  StyleUsed(TexChars)	  
+	  StyleUsed(TexChars)
 	  );
    PrintPackageList();
 #ifndef Registered
@@ -416,7 +416,7 @@ void ShowHelp(void)
 
 void ShowCopyright(void)
 {
-  puts(_("\nThis program and related files are copyright (c) 1996-2023 Fojtik Jaroslav."
+  puts(_("\nThis program and related files are copyright (c) 1996-2024 Fojtik Jaroslav."
 	 "\nThis code can be redistributed and/or modified under the terms of the GNU"
 	 "\nGeneral Public License available from: http://www.gnu.org/copyleft/gpl.html."
 	 "\nWith one additional requirement:"
