@@ -17,8 +17,8 @@
 
 #define LineLength   80		/* Split lines after more than LineLength charcters */
 
-#define VersionWP2L	"4.4"
-#define VersionDate	"26 Nov 2023"  /* day (space) month (space) full year */
+#define VersionWP2L	"4.6"
+#define VersionDate	"25 Apr 2024"  /* day (space) month (space) full year */
 
 
 /** Constants for a flag InputPS */
@@ -109,10 +109,13 @@ void LoadConfigFile(const char *filename);
 int CheckFileFormat(FILE *wpd,FFormatDetect & FilForD);
 
 /* Variables for Convert_first_pass: */
-#define NormalText      0
-#define HeaderText      1
-#define CharsOnly       2
-#define Nothing         3
+typedef enum
+{
+    NormalText =  0,
+    HeaderText,
+    CharsOnly,
+    Nothing
+} E_OBJECT_FILTER;
 
 
 #define MaxAttributes 17	/*the capacity of attribute stack*/
